namespace Eco.Mods.TechTree
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using Eco.Gameplay.Blocks;
    using Eco.Gameplay.Components;
    using Eco.Gameplay.DynamicValues;
    using Eco.Gameplay.Items;
    using Eco.Gameplay.Objects;
    using Eco.Gameplay.Players;
    using Eco.Gameplay.Skills;
    using Eco.Gameplay.Systems;
    using Eco.Gameplay.Systems.TextLinks;
    using Eco.Core.Items;
    using Eco.Shared.Localization;
    using Eco.Shared.Serialization;
    using Eco.Shared.Utils;
    using Eco.World;
    using Eco.World.Blocks;
    using Eco.Gameplay.Pipes;

    [RequiresSkill(typeof(FarmingSkill), 1)]
    public partial class HedgeRecipe : RecipeFamily
    {
        public HedgeRecipe()
        {
            var recipe = new Recipe();
            recipe.Init(
                "GardenHedges",
                Localizer.DoStr("Garden Hedges"),
                new List<IngredientElement>
                {
                    new IngredientElement(typeof(PlantFibersItem), 6, typeof(FarmingSkill), typeof(FarmingLavishResourcesTalent)),
                    new IngredientElement(typeof(WoodPulpItem), 1, true),
                },
                new List<CraftingElement>
                {
                    new CraftingElement<HedgeItem>()
                });
            this.Recipes = new List<Recipe> { recipe };
            this.ExperienceOnCraft = 1;
            this.LaborInCalories = CreateLaborInCaloriesValue(40, typeof(FarmingSkill));
            this.CraftMinutes = CreateCraftTimeValue(typeof(HedgeRecipe), 1.5f, typeof(FarmingSkill), typeof(FarmingFocusedSpeedTalent), typeof(FarmingParallelSpeedTalent));
            this.ModsPreInitialize();
            this.Initialize(Localizer.DoStr("Garden Hedges"), typeof(HedgeRecipe));
            this.ModsPostInitialize();
            CraftingComponent.AddRecipe(typeof(FarmersTableObject), this);
        }

        /// <summary>Hook for mods to customize RecipeFamily before initialization. You can change recipes, xp, labor, time here.</summary>
        partial void ModsPreInitialize();
        /// <summary>Hook for mods to customize RecipeFamily after initialization, but before registration. You can change skill requirements here.</summary>
        partial void ModsPostInitialize();
    }

    [Serialized]
    [Solid, Wall, Constructed,BuildRoomMaterialOption]
    [BlockTier(0)]                                          
    [DoesntEncase]                                          
    [RequiresSkill(typeof(GlassworkingSkill), 1)]      
    public partial class HedgeBlock : Block, IRepresentsItem   
    {
        public Type RepresentedItemType { get { return typeof(HedgeItem); } }  
    }

    [Serialized]
    [LocDisplayName("Garden Hedges")]
    [MaxStackSize(20)]                           
    [Weight(1000)]      
    [Ecopedia("Blocks", "Building Materials", createAsSubPage: true, display: InPageTooltip.DynamicTooltip)]                    
    [Currency][Tag("Currency")]                              
    [Tag("Constructable", 1)]                         
    [Tier(0)]                                      
    public partial class HedgeItem : BlockItem<HedgeBlock>
    {
        public override LocString DisplayNamePlural { get { return Localizer.DoStr("Garden Hedges"); } } 
        public override LocString DisplayDescription { get { return Localizer.DoStr("Garden Hedges"); } }

        public override bool CanStickToWalls { get { return false; } }  

        private static Type[] blockTypes = new Type[] {
            typeof(HedgeCubeStacked1Block),
            typeof(HedgeCubeStacked2Block),
            typeof(HedgeCubeStacked3Block),
            typeof(HedgeCubeStacked4Block)
        };
        public override Type[] BlockTypes { get { return blockTypes; } }
    }

    [Serialized, Solid] public class HedgeCubeStacked1Block : PickupableBlock { }
    [Serialized, Solid] public class HedgeCubeStacked2Block : PickupableBlock { }
    [Serialized, Solid] public class HedgeCubeStacked3Block : PickupableBlock { }
    [Serialized, Solid,Wall] public class HedgeCubeStacked4Block : PickupableBlock { } //Only a wall if it's all 4 Glass

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeCubeFormType), typeof(HedgeItem))]
    public partial class HedgeCubeBlock :
        Block, IRepresentsItem
    {
        public Type RepresentedItemType { get { return typeof(HedgeItem); } }
    }
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHemiSphereFormType), typeof(HedgeItem))]
    public partial class HedgeHemiSphereBlock :
        Block, IRepresentsItem
    {
        public Type RepresentedItemType { get { return typeof(HedgeItem); } }
    }
    //--------------------HedgeQuarterSphere----------------------------//
    
    [RotatedVariants(typeof(HedgeQuarterSphereBlock), typeof(HedgeQuarterSphere90Block), typeof(HedgeQuarterSphere180Block), typeof(HedgeQuarterSphere270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeQuarterSphereFormType), typeof(HedgeItem))]
    public partial class HedgeQuarterSphereBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeQuarterSphere90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeQuarterSphere180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeQuarterSphere270Block : Block
    { }
    //--------------------------------------------------------------------//
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeCylinderFormType), typeof(HedgeItem))]
    public partial class HedgeCylinderBlock :
        Block, IRepresentsItem
    {
        public Type RepresentedItemType { get { return typeof(HedgeItem); } }
    }
    //--------------------HedgeHalfCylinder--V----------------------------//
    

    [RotatedVariants(typeof(HedgeHalfCylinderVBlock), typeof(HedgeHalfCylinderV90Block), typeof(HedgeHalfCylinderV180Block), typeof(HedgeHalfCylinderV270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderVFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderVBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderV90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderV180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderV270Block : Block
    { }
    //--------------------------------------------------------------------//
    //--------------------HedgeHalfCylinder--G----------------------------//
    
    [RotatedVariants(typeof(HedgeHalfCylinderGBlock), typeof(HedgeHalfCylinderG90Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderGFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderGBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderG90Block : Block
    { }
    //--------------------------------------------------------------------//
    //--------------------HedgeTopAngleCylinder----------------------------//
    
    [RotatedVariants(typeof(HedgeTopAngleCylinderBlock), typeof(HedgeTopAngleCylinder90Block), typeof(HedgeTopAngleCylinder180Block), typeof(HedgeTopAngleCylinder270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeTopAngleCylinderFormType), typeof(HedgeItem))]
    public partial class HedgeTopAngleCylinderBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTopAngleCylinder90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTopAngleCylinder180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTopAngleCylinder270Block : Block
    { }
    //--------------------------------------------------------------------//
    //--------------------HedgeBottomAngleCylinder----------------------------//

    [RotatedVariants(typeof(HedgeBottomAngleCylinderBlock), typeof(HedgeBottomAngleCylinder90Block), typeof(HedgeBottomAngleCylinder180Block), typeof(HedgeBottomAngleCylinder270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeBottomAngleCylinderFormType), typeof(HedgeItem))]
    public partial class HedgeBottomAngleCylinderBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeBottomAngleCylinder90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeBottomAngleCylinder180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeBottomAngleCylinder270Block : Block
    { }
    //--------------------------------------------------------------------//
    //----------------------HedgeHalfCylinderT-----------------------------//
    [RotatedVariants(typeof(HedgeHalfCylinderTBlock), typeof(HedgeHalfCylinderT90Block), typeof(HedgeHalfCylinderT180Block), typeof(HedgeHalfCylinderT270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderTFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderTBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderT90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderT180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderT270Block : Block
    { }
    //--------------------------------------------------------------------//
    
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderXFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderXBlock :
    Block, IRepresentsItem
    {
        public Type RepresentedItemType { get { return typeof(HedgeItem); } }
    }
    //----------------------HedgeTriangle-----------------------------//
    [RotatedVariants(typeof(HedgeTriangleBlock), typeof(HedgeTriangle90Block), typeof(HedgeTriangle180Block), typeof(HedgeTriangle270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeTriangleFormType), typeof(HedgeItem))]
    public partial class HedgeTriangleBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTriangle90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTriangle180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeTriangle270Block : Block
    { }
    //--------------------------------------------------------------------//
    //----------------------HedgeHalfCylinderTurn-----------------------------//
    [RotatedVariants(typeof(HedgeHalfCylinderTurnBlock), typeof(HedgeHalfCylinderTurn90Block), typeof(HedgeHalfCylinderTurn180Block), typeof(HedgeHalfCylinderTurn270Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderTurnFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderTurnBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderTurn90Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderTurn180Block : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderTurn270Block : Block
    { }
    //--------------------------------------------------------------------//
    //----------------------HedgeHalfCylinderGR-----------------------------//
    [RotatedVariants(typeof(HedgeHalfCylinderGRBlock), typeof(HedgeHalfCylinderGR90Block))]
    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    [IsForm(typeof(HedgeHalfCylinderGRFormType), typeof(HedgeItem))]
    public partial class HedgeHalfCylinderGRBlock : Block
    { }

    [Serialized]
    [Wall, Constructed, Solid, BuildRoomMaterialOption]
    [BlockTier(0)]
    public partial class HedgeHalfCylinderGR90Block : Block
    { }

    
    //--------------------------------------------------------------------//


}